﻿# -*- coding: <utf-8> -*-

import os, sys, random

CODING = "UTF-8" 

class File:
	def __init__(self, name, filetypes=None):
		self.full = name
		if os.path.isfile(self.full):
			(self.path, self.name) = os.path.split(self.full)
			(self.name, self.ext) = os.path.splitext(self.name)
			self.ext = self.ext[1:]
			self.type = 'file'
			if filetypes:
				for ext in filetypes:
					isok=False
					if self.ext.lower() == ext.lower():
						isok = True
						break
				if isok == False:
					self.type = None
		elif os.path.isdir(self.full):
			self.path = self.full
			(spam, self.name) = os.path.split(self.full)
			self.ext = ''
			self.type = 'dir'
		else:
			self.path = ''
			self.name = ''
			self.ext = ''
			self.type = None
			

class List:
	def __init__(self):
		self.data = []
		self.start = 0
		self.tmp_start = 0
		self.marker = 0
		self.tmp_marker = 0
		self.path = ""
		self.main_path = ""
	
	def marked_index(self):
		return self.start + self.marker

	def marked(self):
		if self.data:
			return self.data[self.start + self.marker].name
		else:
			return None

	def marked_path(self):
		if self.data: 	return self.data[self.start + self.marker].full
		else:			return self.path
	
	def isdir(self):
		if self.data[self.start + self.marker].type == 'dir':
			return True
		else:
			return False
		
	def isfile(self):
		if self.data[self.start + self.marker].type == 'file':
			return True
		else:
			return False
	
	def next(self, max):
		if self.start + self.marker < len(self.data)-1:
			if self.marker < max-1:
				self.marker += 1
			else:
				self.start += max
				self.marker = 0
		else:
			self.start = 0
			self.marker = 0

	def prev(self, max):
		if self.marker + self.start > 0:
			if self.marker > 0:
				self.marker -= 1
			else:
				if self.start - max < 0:
					self.start = 0
				else:
					self.start -= max
				self.marker = max - 1
		else:
			if len(self.data) > max:
				self.start = len(self.data) - max
				self.marker = max - 1
			else:
				self.marker = len(self.data) - 1

	def pgdown(self, max):
		if self.start < len(self.data) - max * 2:
			self.start += max
		else:
			self.start = 0

	def pgup(self, max):
		if len(self.data) > max - 1:
			if self.start > max * 2:
				self.start -= max
			else:
				self.start = len(self.data) - max

	def generate(self, filetypes):
		self.data = []
		inp = os.listdir(self.path)	
		inp.sort()
		for line in inp:
			file = File(os.path.join(self.path, line), filetypes)
			if file.type != None:
				self.data.append(file)

	def generate2(self, filelist):
		self.clear = []
		for line in filelist:
			file = File(os.path.join(self.path, line))
			if file.type != None:
				self.data.append(file)
	
	def destroy(self):
		self.data = []
		self.start = 0
		self.tmp_start = 0
		self.marker = 0
		self.tmp_marker = 0
		self.path = ""
		#self.main_path = ""

	def store(self):
		self.olddata = self.data
		self.oldstart = self.start
		self.oldtmp_start = self.tmp_start
		self.oldmarker = self.marker
		self.oldtmp_marker = self.tmp_marker
		self.oldpath = self.path
		self.oldmain_path = self.main_path

	def restore(self):
		self.data = self.olddata
		self.start = self.oldstart
		self.tmp_start = self.oldtmp_start
		self.marker = self.oldmarker
		self.tmp_marker = self.oldtmp_marker
		self.path = self.oldpath
		self.main_path = self.oldmain_path

	def store2(self):
		self.olddata2 = self.data
		self.oldstart2 = self.start
		self.oldtmp_start2 = self.tmp_start
		self.oldmarker2 = self.marker
		self.oldtmp_marker2 = self.tmp_marker
		self.oldpath2 = self.path
		self.oldmain_path2 = self.main_path

	def restore2(self):
		self.data = self.olddata2
		self.start = self.oldstart2
		self.tmp_start = self.oldtmp_start2
		self.marker = self.oldmarker2
		self.tmp_marker = self.oldtmp_marker2
		self.path = self.oldpath2
		self.main_path = self.oldmain_path2

class Playlist:
	def __init__(self, output):
		if output == 'sdlmixer':
			print 'using sdl_mixer as soundmodule'
			from epm_sdlmixer import Music
		elif output == 'pysonic':
			print 'using pySonic as soundmodule'
			from epm_pysonic import Music
		self.tracks = []
		self.pos = 0
		self.out = Music()

	def isplaying(self):
		return self.out.isplaying()
	
	def play(self):
		if self.tracks:
			print 'trying to open ', self.tracks[self.pos]
			if self.out.play(self.tracks[self.pos]):
				self.save()
			else:
				if len(self.tracks) != 1:
					print 'ERROR: can not open ', self.tracks[self.pos], ' skipping it...'
					self.tracks.remove(self.tracks[self.pos])
					self.play()
				else:
					print 'ERROR: can not open ', self.tracks[self.pos], ' stoping playback...'
					self.stop()
		
	def pause(self):
		if self.isplaying:
			self.out.pause()

	def playpause(self):
		if self.out.paused:
			print 'resume playing'
			self.out.play()
		else:
			print 'pause'
			self.out.pause()
		return self.out.paused
	
	def time(self):
		if self.out.time():
			return self.out.time()
		else:
			return ' '
	
	def track(self, split=False, next=False):
		if self.tracks:
			if next:
				if len(self.tracks) == 1:
					txt = ' '
				elif len(self.tracks) == self.pos + 1:
					txt = self.tracks[0]
				else:
					txt = self.tracks[self.pos + 1]
			else:
				txt = self.tracks[self.pos]
			if split:
				trash, txt = os.path.split(txt)
				txt, trash = os.path.splitext(txt)
			return txt
		else:
			return None
	
	def stop(self):
		self.tracks = []
		self.pos = 0
		if self.out.isplaying():
			print 'stop'
			self.out.playing = False
			self.out.stop()
			if os.path.isfile('epmedia.lst'):
				try:
					os.unlink('epmedia.lst')
				except OSError:
					print 'ERROR: could not delete epmedia.lst (write protected?)' 
	
	def next(self):
		if self.tracks:
			print 'skip'
			if self.pos == len(self.tracks) - 1:
				self.pos = 0
			else:
				self.pos = self.pos + 1
			self.play()
		
	def prev(self):
		if self.tracks:
			print 'replay'
			if self.pos == 0:
				self.pos = len(self.tracks) - 1
			else:
				self.pos = self.pos - 1
			self.play()
		
	def add(self, name):
		self.tracks.append(name)
		
	def add_dir(self, name, filetypes, selected=None, keep=False):
		old_tracks = self.tracks
		if keep == False:
			self.tracks = []
			self.pos = 0
		inp = os.listdir(name)
		for line in inp:
			(shortname, ext) = os.path.splitext(line)
			for lin in filetypes:
				if ext[1:] == lin:
					self.tracks.append(os.path.join(name, line))
					break
		if self.tracks != [] and keep == False:
			if selected:
				self.pos=self.tracks.index(selected)
			if self.out.isplaying():
				self.out.stop()
			self.play()
		else:
			self.tracks = old_tracks

	def super_add(self, name, filetypes, keep=False):
		old_tracks = self.tracks
		if keep == False:
			self.tracks = []
			self.pos = 0
		sadd = Superadd(name, filetypes)
		if sadd.files != [] and keep == False:
			self.tracks = sadd.files
			if self.out.isplaying():
				self.out.stop()
			self.play()
		else:
			self.tracks = old_tracks

	def shuffle(self):
		if self.tracks:
			self.out.stop()
			random.shuffle(self.tracks)
			self.pos = 0
			self.play()

	def save(self):
		if self.tracks:
			out_file = open('epmedia.lst','w')
			out_file.write(str(self.pos) + '\n')
			for file in self.tracks:
				try:
					out_file.write(file + '\n')
				except:
					pass
			out_file.close()
		
	def load(self, file='epmedia.lst'):
		if os.path.isfile(file):
			in_file = open(file,'r')
			self.tracks = []
			self.pos = 0
			i = 0
			for line in in_file:
				if i == 0 and file == 'epmedia.lst':
					self.pos = int(line)
				elif line[1] != '#':
					if os.path.isfile(line[:-1]):
						self.tracks.append(str(line[:-1]))
				i += 1
			if self.tracks:
				self.play()

class Superadd:
	def __init__(self, path, ft=None):
		self.files = []
		try:
			self.searchdir(path, ft)
		except:
			print 'skipping ', path
	def searchdir(self, path, ft=None):
		for line in os.listdir(path):
			if os.path.isdir(os.path.join(path,line)):
				self.searchdir(os.path.join(path,line), ft)
			else:
				if ft:
					for ext in ft:
						(shortname, ex) = os.path.splitext(line)
						if ex[1:] == ext:
							self.files.append(os.path.join(path,line))
				else:
					self.files.append(os.path.join(path,line))
		return self.files.sort()